#include "stdafx.h"
#include "PT1SendCtrlCmdUtil.h"

#define CONNECT_TIMEOUT 30*1000

DWORD Send(HANDLE hPipe, CMD_STREAM &stream)
{
	DWORD dwWrite = 0;
	DWORD dwSendNum = 0;
	while(dwSendNum < stream.dwSize ){
		DWORD dwSendSize = 0;
		if( stream.dwSize - dwSendNum < SEND_BUFF_SIZE ){
			dwSendSize = stream.dwSize - dwSendNum;
		}else{
			dwSendSize = SEND_BUFF_SIZE;
		}
		if( WriteFile(hPipe, stream.bData+dwSendNum, dwSendSize, &dwWrite, NULL ) == FALSE ){
			return CMD_ERR;
		}
		dwSendNum+=dwWrite;
	}
	return CMD_SUCCESS;
}

DWORD Recv(HANDLE hPipe, CMD_STREAM &stream) {
	DWORD dwRead = 0;
	DWORD dwReadNum = 0;

	stream.bData = NEW_DATA_BUFF(stream.dwSize);
	while(dwReadNum < stream.dwSize ){
		DWORD dwReadSize = 0;
		if( stream.dwSize - dwReadNum < RES_BUFF_SIZE ){
			dwReadSize = stream.dwSize-dwReadNum;
		}else{
			dwReadSize = RES_BUFF_SIZE;
		}
		if( ReadFile(hPipe, stream.bData+dwReadNum, dwReadSize, &dwRead, NULL ) == FALSE ){
			return CMD_ERR;
		}
		dwReadNum+=dwRead;
	}
	return CMD_SUCCESS;
}

DWORD WaitConnect(LPCWSTR lpwszName)
{
	HANDLE hEvent = _CreateEvent(FALSE, FALSE, lpwszName);
	if( hEvent == NULL ){
		return CMD_ERR;
	}
	DWORD dwRet = CMD_SUCCESS;
	if(WaitForSingleObject(hEvent, CONNECT_TIMEOUT) != WAIT_OBJECT_0){
		dwRet = CMD_ERR_TIMEOUT;
	}
	CloseHandle(hEvent);
	
	return dwRet;
}

DWORD PipeClient::SendDefCmd(LPCWSTR lpwszEventName, LPCWSTR lpwszPipeName, CMD_STREAM* pstSend, CMD_STREAM* pstRes)
{
	DWORD dwRet = CMD_SUCCESS;
	if( pstSend == NULL || pstRes == NULL ){
		return CMD_ERR_INVALID_ARG;
	}

	dwRet = WaitConnect(lpwszEventName);
	if( dwRet != CMD_SUCCESS ){
		return dwRet;
	}
	HANDLE hPipe = _CreateFile(lpwszPipeName, GENERIC_READ|GENERIC_WRITE, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if( hPipe == INVALID_HANDLE_VALUE ){
		return CMD_ERR_CONNECT;
	}
	DWORD dwWrite = 0;
	DWORD dwRead = 0;

	if( WriteFile(hPipe, pstSend, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
		return CMD_ERR;
	}
	if( pstSend->dwSize > 0 ){
		if( pstSend->bData == NULL ){
			return CMD_ERR_INVALID_ARG;
		}
		dwRet = Send( hPipe, *pstSend );
		if( dwRet != CMD_SUCCESS ) {
			return dwRet;
		}
	}
	if( ReadFile(hPipe, pstRes, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
		return CMD_ERR;
	}
	if( pstRes->dwSize > 0 ){
		dwRet = Recv( hPipe, *pstRes );
		if( dwRet != CMD_SUCCESS ) {
			return dwRet;
		}
	}
	CloseHandle(hPipe);

	return pstRes->dwParam;
}

HANDLE PipeClient::SendEnumCmdFirst(LPCWSTR lpwszEventName, LPCWSTR lpwszPipeName, CMD_STREAM* pstSend, CMD_STREAM* pstRes)
{
	DWORD dwRet = CMD_SUCCESS;
	if( pstSend == NULL || pstRes == NULL ){
		return INVALID_HANDLE_VALUE;
	}

	dwRet = WaitConnect(lpwszEventName);
	if( dwRet != CMD_SUCCESS ){
		return INVALID_HANDLE_VALUE;
	}
	HANDLE hPipe = _CreateFile(lpwszPipeName, GENERIC_READ|GENERIC_WRITE, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if( hPipe == INVALID_HANDLE_VALUE ){
		return INVALID_HANDLE_VALUE;
	}
	DWORD dwWrite = 0;
	DWORD dwRead = 0;

	if( WriteFile(hPipe, pstSend, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
		return INVALID_HANDLE_VALUE;
	}
	if( pstSend->dwSize > 0 ){
		if( pstSend->bData == NULL ){
			CloseHandle( hPipe );
			return INVALID_HANDLE_VALUE;
		}
		dwRet = Send( hPipe, *pstSend );
		if( dwRet != CMD_SUCCESS ) {
			CloseHandle( hPipe );
			return INVALID_HANDLE_VALUE;
		}
	}
	if( ReadFile(hPipe, pstRes, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
		CloseHandle( hPipe );
		return INVALID_HANDLE_VALUE;
	}
	if( pstRes->dwSize > 0 ){
		dwRet = Recv( hPipe, *pstRes );
		if( dwRet != CMD_SUCCESS ) {
			CloseHandle( hPipe );
			return INVALID_HANDLE_VALUE;
		}
	}
	return hPipe;
}

DWORD PipeClient::SendEnumCmdNext(HANDLE hPipe, CMD_STREAM* pstSend, CMD_STREAM* pstRes)
{
	DWORD dwRet = CMD_SUCCESS;
	if( pstSend == NULL || pstRes == NULL ){
		return CMD_ERR_INVALID_ARG;
	}

	DWORD dwWrite = 0;
	DWORD dwRead = 0;

	if( WriteFile(hPipe, pstSend, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
		return CMD_ERR;
	}
	if( pstSend->dwSize > 0 ){
		if( pstSend->bData == NULL ){
			return CMD_ERR_INVALID_ARG;
		}
		dwRet = Send( hPipe, *pstSend );
		if( dwRet != CMD_SUCCESS ) {
			return dwRet;
		}
	}
	if( ReadFile(hPipe, pstRes, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
		return CMD_ERR;
	}
	if( pstRes->dwSize > 0 ){
		dwRet = Recv( hPipe, *pstRes );
		if( dwRet != CMD_SUCCESS ) {
			return dwRet;
		}
	}

	return pstRes->dwParam;
}

DWORD PipeClient::SendCloseExe()
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_CLOSE_EXE;

	DWORD dwRet = SendDefCmd(CMD_PT1_CTRL_EVENT_WAIT_CONNECT, CMD_PT1_CTRL_PIPE, &stSend, &stRes);

	return dwRet;
}

DWORD PipeClient::SendOpenTuner(BOOL bSate, int* piID, DWORD *pUnique)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_OPEN_TUNER;
	CreateDefStream(bSate, &stSend);

	DWORD dwRet = SendDefCmd(CMD_PT1_CTRL_EVENT_WAIT_CONNECT, CMD_PT1_CTRL_PIPE, &stSend, &stRes);
	if( dwRet == CMD_SUCCESS ){
		CopyDefData2((DWORD*)piID, pUnique, stRes.bData);
	}

	return dwRet;
}

DWORD PipeClient::SendOpenTuner2(BOOL bSate, int iTunerID, int* piID, DWORD *pUnique)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_OPEN_TUNER2;
	CreateDefStream2(bSate, iTunerID, &stSend);

	DWORD dwRet = SendDefCmd(CMD_PT1_CTRL_EVENT_WAIT_CONNECT, CMD_PT1_CTRL_PIPE, &stSend, &stRes);
	if( dwRet == CMD_SUCCESS ){
		CopyDefData2((DWORD*)piID, pUnique, stRes.bData);
	}

	return dwRet;
}

DWORD PipeClient::SendCloseTuner(int iID)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_CLOSE_TUNER;
	CreateDefStream(iID, &stSend);

	DWORD dwRet = SendDefCmd(CMD_PT1_CTRL_EVENT_WAIT_CONNECT, CMD_PT1_CTRL_PIPE, &stSend, &stRes);

	return dwRet;
}

DWORD PipeClient::SendSetCh(int iID, DWORD dwCh, DWORD dwTSID)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_SET_CH;
	CreateDefStream3(iID, dwCh, dwTSID, &stSend);

	DWORD dwRet = SendDefCmd(CMD_PT1_CTRL_EVENT_WAIT_CONNECT, CMD_PT1_CTRL_PIPE, &stSend, &stRes);

	return dwRet;
}

DWORD PipeClient::SendGetSignal(int iID, DWORD* pdwCn100)
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_GET_SIGNAL;
	CreateDefStream(iID, &stSend);

	DWORD dwRet = SendDefCmd(CMD_PT1_CTRL_EVENT_WAIT_CONNECT, CMD_PT1_CTRL_PIPE, &stSend, &stRes);
	if( dwRet == CMD_SUCCESS ){
		CopyDefData(pdwCn100, stRes.bData);
	}

	return dwRet;
}

DWORD PipeClient::SendSendData(int iID, BYTE* pbData, DWORD pdwSize )
{
	CMD_STREAM stSend;
	CMD_STREAM stRes;

	stSend.dwParam = CMD_SEND_DATA;

	wstring strEvent;
	wstring strPipe;
	Format(strEvent, L"%s%d", CMD_PT1_DATA_EVENT_WAIT_CONNECT, iID);
	Format(strPipe, L"%s%d", CMD_PT1_DATA_PIPE, iID);

	DWORD dwRet = SendDefCmd(strEvent.c_str(), strPipe.c_str(), &stSend, &stRes);
	if( dwRet == CMD_SUCCESS ){
		if( pdwSize >= stRes.dwSize ){
			memcpy(pbData, stRes.bData, stRes.dwSize);
		}else{
			return CMD_ERR_INVALID_ARG;
		}
	}

	return dwRet;
}

DWORD CMemClient::Open(int iID, DWORD unique )
{
	wstring serverName;
	Format(serverName, L"%s%08x%08x", CMD_PT1_DATA, iID, unique );

	wstring eventConnectName;
	Format( eventConnectName, L"Global\\%sEventConnect", serverName.c_str() );

	DWORD dwRet = CMD_SUCCESS;
	dwRet = WaitConnect( eventConnectName.c_str() );
	if( dwRet != CMD_SUCCESS ){
		return dwRet;
	}

	wstring eventCallName;
	Format( eventCallName, L"Global\\%sEventCall", serverName.c_str() );
	m_hEventCall = _CreateEvent(TRUE, FALSE, eventCallName.c_str());

	wstring eventReturnName;
	Format( eventReturnName, L"Global\\%sEventReturn", serverName.c_str() );
	m_hEventReturn = _CreateEvent(TRUE, FALSE, eventReturnName.c_str());

	wstring mappingName;
	Format( mappingName, L"Global\\%sMapping", serverName.c_str() );
	//hMapping = ::CreateFileMapping( INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, 1024*1024, mappingName.c_str() );
	m_hMapping = OpenFileMapping( FILE_MAP_ALL_ACCESS, TRUE, mappingName.c_str() );
	m_buf = (BYTE *)MapViewOfFile( m_hMapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);
	return dwRet;
}

void CMemClient::Close() 
{
	UnmapViewOfFile( m_buf );
	CloseHandle( m_hMapping );
	CloseHandle( m_hEventReturn );
	CloseHandle( m_hEventCall );
}

DWORD CMemClient::GetTSBlock()// void *buf, size_t bufSize )
{
	SetEvent( m_hEventCall );
	DWORD dwRes = WaitForSingleObject( m_hEventReturn, 2000);
	
	if( dwRes == WAIT_OBJECT_0 ) {
		return *(DWORD *)m_buf;
	} else {
		return 0;
	}
}

