#include "StdAfx.h"
#include "PipeServer.h"
#include <process.h>

#define PIPE_TIMEOUT 500

CPipeServer::CPipeServer(void)
{
	m_pCmdProc = NULL;
	m_pParam = NULL;
	m_strEventName = L"";
	m_strPipeName = L"";

	m_hStopEvent = _CreateEvent(FALSE, FALSE, NULL);
	m_hThread = NULL;
}

CPipeServer::~CPipeServer(void)
{
	if( m_hThread != NULL ){
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
	::CloseHandle(m_hStopEvent);
	m_hStopEvent = NULL;
}

BOOL CPipeServer::StartServer(LPCWSTR lpwszEventName, LPCWSTR lpwszPipeName, CMD_CALLBACK_PROC pfnCmdProc, void* pParam, int iThreadPriority)
{
	if( pfnCmdProc == NULL || pParam == NULL || lpwszEventName == NULL || lpwszPipeName == NULL ){
		return FALSE;
	}
	if( m_hThread != NULL ){
		return FALSE;
	}
	m_pCmdProc = pfnCmdProc;
	m_pParam = pParam;
	m_strEventName = lpwszEventName;
	m_strPipeName = lpwszPipeName;
	m_iThreadPriority = iThreadPriority;

	ResetEvent(m_hStopEvent);
	m_hThread = (HANDLE)_beginthreadex(NULL, 0, ServerThread, (LPVOID)this, CREATE_SUSPENDED, NULL);
	ResumeThread(m_hThread);

	return TRUE;
}

void CPipeServer::StopServer()
{
	if( m_hThread != NULL ){
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
	
	if (m_hPipe != NULL) {
		FlushFileBuffers(m_hPipe);
		DisconnectNamedPipe(m_hPipe);
		CloseHandle(m_hPipe);
		m_hPipe = NULL;
	}
	if( m_hEventCmdWait != NULL ) {
		CloseHandle(m_hEventCmdWait);
		m_hEventCmdWait = NULL;
	}
	if( m_hEventConnect != NULL ) {
		CloseHandle(m_hEventConnect);
		m_hEventConnect = NULL;
	}

	
}

UINT WINAPI CPipeServer::ServerThread(LPVOID pParam)
{
	CPipeServer* pSys = (CPipeServer*)pParam;
	return pSys->ServerThread();
}
UINT WINAPI CPipeServer::ServerThread()
{

	HANDLE hCurThread = GetCurrentThread();
	SetThreadPriority(hCurThread, m_iThreadPriority);

	HANDLE hEventArray[2];
	OVERLAPPED stOver;

	m_hEventCmdWait = _CreateEvent(FALSE, TRUE, CMD_CTRL_EVENT_WAIT);
	m_hEventConnect = _CreateEvent(FALSE, FALSE, m_strEventName.c_str());
	hEventArray[0] = m_hStopEvent;
	hEventArray[1] = _CreateEvent(FALSE, FALSE, NULL);
	m_hPipe = NULL;

	while(1){
		if( m_hPipe == NULL ){
			while(1){
				m_hPipe = _CreateNamedPipe(m_strPipeName.c_str(), PIPE_ACCESS_DUPLEX | FILE_FLAG_WRITE_THROUGH | FILE_FLAG_OVERLAPPED,
				PIPE_TYPE_BYTE, 1,
				SEND_BUFF_SIZE, RES_BUFF_SIZE, PIPE_TIMEOUT);
				if( GetLastError() == ERROR_PIPE_BUSY ){
					Sleep(100);
				}else{
					break;
				}
			}

			ZeroMemory(&stOver, sizeof(OVERLAPPED));
			stOver.hEvent = hEventArray[1];
			ConnectNamedPipe(m_hPipe, &stOver);
			SetEvent(m_hEventConnect);
		}

		DWORD dwRes = WaitForMultipleObjects(2, hEventArray, FALSE, INFINITE);
		if( dwRes == WAIT_OBJECT_0 ){
			//STOP
			CloseHandle(m_hPipe);
			m_hPipe = NULL;
			break;
		}else if( dwRes == WAIT_OBJECT_0+1 ){
			//ق̃T[o[ŏH
//			WaitForSingleObject(hEventCmdWait, INFINITE);
			//R}hM
			if( m_pCmdProc != NULL){
				CMD_STREAM stCmd;
				CMD_STREAM stRes;
				DWORD dwRead = 0;
				DWORD dwWrite = 0;
				do{
					if( ReadFile(m_hPipe, &stCmd, sizeof(DWORD)*2, &dwRead, NULL ) == FALSE ){
						break;
					}
					if( stCmd.dwSize > 0 ){
						stCmd.bData = NEW_DATA_BUFF(stCmd.dwSize);
						DWORD dwReadNum = 0;
						while(dwReadNum < stCmd.dwSize ){
							DWORD dwReadSize = 0;
							if( stCmd.dwSize-dwReadNum < SEND_BUFF_SIZE ){
								dwReadSize = stCmd.dwSize-dwReadNum;
							}else{
								dwReadSize = SEND_BUFF_SIZE;
							}
							if( ReadFile(m_hPipe, stCmd.bData+dwReadNum, dwReadSize, &dwRead, NULL ) == FALSE ){
								break;
							}
							dwReadNum+=dwRead;
						}
						if( dwReadNum < stCmd.dwSize ){
							break;
						}
					}

					m_pCmdProc(m_pParam, &stCmd, &stRes);

					if( WriteFile(m_hPipe, &stRes, sizeof(DWORD)*2, &dwWrite, NULL ) == FALSE ){
						break;
					}
					if( stRes.dwSize > 0 ){
						if( stRes.bData == NULL ){
							break;
						}
						DWORD dwSendNum = 0;
						while(dwSendNum < stRes.dwSize ){
							DWORD dwSendSize = 0;
							if( stRes.dwSize-dwSendNum < RES_BUFF_SIZE ){
								dwSendSize = stRes.dwSize-dwSendNum;
							}else{
								dwSendSize = RES_BUFF_SIZE;
							}
							if( WriteFile(m_hPipe, stRes.bData+dwSendNum, dwSendSize, &dwWrite, NULL ) == FALSE ){
								break;
							}
							dwSendNum+=dwWrite;
						}
						if( dwSendNum < stRes.dwSize ){
							break;
						}
					}
					DELETE_DATA_BUFF( stRes.bData, stRes.dwSize );
				}while(stRes.dwParam == CMD_NEXT); //Emunp̌JԂ
			}
			FlushFileBuffers(m_hPipe);
			DisconnectNamedPipe(m_hPipe);
			CloseHandle(m_hPipe);
			m_hPipe = NULL;
//			SetEvent(hEventCmdWait);
		}
	}

	CloseHandle(hEventArray[1]);

	CloseHandle(m_hEventConnect);
	m_hEventConnect = NULL;
	CloseHandle(m_hEventCmdWait);
	m_hEventCmdWait = NULL;
	return 0;
}


