#include "stdafx.h"

HANDLE _CreateEvent(BOOL bManualReset, BOOL bInitialState, LPCTSTR lpName)
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateEvent(&sa, bManualReset, bInitialState, lpName);
}

HANDLE _CreateFile( LPCTSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, DWORD dwCreationDisposition, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile )
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateFile( lpFileName, dwDesiredAccess, dwShareMode, &sa, dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile );
}

HANDLE _CreateMutex( BOOL bInitialOwner, LPCTSTR lpName )
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateMutex( &sa, bInitialOwner, lpName );
}

HANDLE _CreateFileMapping( HANDLE hFile, DWORD flProtect, DWORD dwMaximumSizeHigh, DWORD dwMaximumSizeLow, LPCTSTR lpName)
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateFileMapping( hFile, &sa, flProtect, dwMaximumSizeHigh, dwMaximumSizeLow, lpName);
}

HANDLE _CreateNamedPipe( LPCTSTR lpName, DWORD dwOpenMode, DWORD dwPipeMode, DWORD nMaxInstances, DWORD nOutBufferSize, DWORD nInBufferSize, DWORD nDefaultTimeOut )
{
	SECURITY_DESCRIPTOR sd;
	SECURITY_ATTRIBUTES sa;
	 
	memset(&sd,0,sizeof(sd));
	InitializeSecurityDescriptor(&sd,SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
	memset(&sa,0,sizeof(sa));
	sa.nLength=sizeof(sa);
	sa.lpSecurityDescriptor=&sd;

	return ::CreateNamedPipe( lpName, dwOpenMode, dwPipeMode, nMaxInstances, nOutBufferSize, nInBufferSize, nDefaultTimeOut, &sa );
}


class CS {
	CRITICAL_SECTION criticalSection;
public:
	CS() {
		InitializeCriticalSection( &criticalSection );
	}
	~CS() {
		DeleteCriticalSection( &criticalSection );
	}
	void Enter() {
		EnterCriticalSection( &criticalSection );
	}
	void Leave() {
		LeaveCriticalSection( &criticalSection );
	}
} cs;

typedef std::vector<BYTE *> BufList;
typedef std::map< int, std::vector<BYTE *> > BufListMap;
BufListMap bufListMap;

BYTE *NEW_DATA_BUFF(int size )
{
	_ASSERT( size > 0 );
	cs.Enter();

	BYTE *ret;
	BufList &bufList = bufListMap[size];
	if( bufList.size() > 0 ) {
		ret = bufList[bufList.size()-1];
		bufList.pop_back();
	} else {
		ret = new BYTE[size];
		//WCHAR str[64];
		//wsprintf( str, L"NEW_DATA_BUFF size=%d\n", size );
		//OutputDebugString( str );
	}
	cs.Leave();
	return ret;
}

void DELETE_DATA_BUFF(BYTE *&buf, int size)
{
	if( size == 0 ) return;
	if( buf == NULL ) return;
	_ASSERT( size > 0 );

	cs.Enter();
	BufList &bufList = bufListMap[size];

	bufList.push_back( buf );
	if( bufList.size() > 64 ) {
		std::sort( bufList.begin(), bufList.end() );
		delete bufList[bufList.size()-1];
		//WCHAR str[64];
		//wsprintf( str, L"DELETE_DATA_BUFF size=%d\n", size );
		//OutputDebugString( str );
		bufList.pop_back();
	}
	buf = NULL;
	cs.Leave();
}

void PURGE_DATA_BUFF()
{
	cs.Enter();
	for( BufListMap::iterator i = bufListMap.begin(); i != bufListMap.end(); ++i ) {
		BufList &list = (*i).second;
		//WCHAR str[64];
		//wsprintf( str, L"Purge size=%d count=%d\n", (*i).first, list.size() );
		//OutputDebugString( str );
		for( BufList::iterator j = list.begin(); j != list.end(); ++j ) {
			delete[] *j;
			*j = NULL;
		}
	}
	cs.Leave();
}