#include "StdAfx.h"
#include "DataIO.h"
#include <process.h>

#define MAX_DATA_BUFF_COUNT 500

CDataIO::CDataIO(void)
{
	VIRTUAL_COUNT = 8;

	m_hStopEvent = _CreateEvent(FALSE, FALSE, NULL);
	m_hThread = NULL;

	m_pcDevice = NULL;

	m_bQuit = FALSE;

	for(int j = 0; j < 4; j++ ) {
		dataIoTs[j] = NULL; 
		m_SetBuff[j] = new BUFF_DATA;
		m_rawPacketCur[j] = 0;
	}
	m_bDMABuff = (BYTE *)_aligned_malloc( READ_BLOCK_SIZE, 64 );
	m_rawPacket = (DWORD (*)[4][64]) _aligned_malloc(4*256,16);
}

CDataIO::~CDataIO(void)
{
	if( m_hThread != NULL ){
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
	if( m_hStopEvent != NULL ){
		::CloseHandle(m_hStopEvent);
		m_hStopEvent = NULL;
	}

	for(int j = 0; j < 4; j++ ) {
		CDataIO::DATAIO_TS *ts = dataIoTs[j];
		if(ts) delete ts;
		SAFE_DELETE(m_SetBuff[j]);
	}
	_aligned_free( m_bDMABuff );

	_aligned_free( m_rawPacket );

}

void CDataIO::ClearBuff(int iID)
{
	int iDevID = iID>>16;
	PT::Device::ISDB enISDB = (PT::Device::ISDB)((iID&0x0000FF00)>>8);
	uint iTuner = iID&0x000000FF;

	int id = GetID( enISDB, iTuner ); 
	if ( dataIoTs[id] )
		dataIoTs[id]->ClearBuff();

	m_SetBuff[id]->dwSetSize = 0;
}

void CDataIO::DATAIO_TS::ClearBuff()
{
	Lock();
	m_dwOverFlowCount = 0;
	while ( !m_Buff.empty() ) {
		SAFE_DELETE( m_Buff.front() );
		m_Buff.pop();
	}
	UnLock();
}

void CDataIO::Run()
{
	if( m_hThread != NULL ){
		return ;
	}
	status enStatus;

	enStatus = m_pcDevice->SetBufferInfo(NULL);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
	PT::Device::BufferInfo bufferInfo;
	bufferInfo.VirtualSize  = VIRTUAL_IMAGE_COUNT;
	bufferInfo.VirtualCount = VIRTUAL_COUNT;
	bufferInfo.LockSize     = LOCK_SIZE;
	enStatus = m_pcDevice->SetBufferInfo(&bufferInfo);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}

	ResetDMA();

	m_bQuit = false;
	m_hThread = (HANDLE)_beginthreadex(NULL, 0, RecvThread, (LPVOID)this, CREATE_SUSPENDED, NULL);
	ResumeThread(m_hThread);
}

void CDataIO::ResetDMA()
{
	status enStatus;

	// DMA ]~
	enStatus = m_pcDevice->SetTransferEnable(false);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}

	mVirtualIndex=0;
	mImageIndex=0;
	mBlockIndex=0;

	// DMA ]ǂ܂Ői񂾂𒲂ׂ邽߁AeubN̖ 0 ŃNA
	for (uint i=0; i<VIRTUAL_COUNT; i++) {
		for (uint j=0; j<VIRTUAL_IMAGE_COUNT; j++) {
			for (uint k=0; k<READ_BLOCK_COUNT; k++) {
				Clear(i, j, k);
			}
		}
	}

	// ]JE^Zbg
	enStatus = m_pcDevice->ResetTransferCounter();
	if( enStatus != PT::STATUS_OK ){
		return ;
	}

	// ]JE^CNg
	for (uint i=0; i<VIRTUAL_IMAGE_COUNT*VIRTUAL_COUNT; i++) {
		enStatus = m_pcDevice->IncrementTransferCounter();
		if( enStatus != PT::STATUS_OK ){
			return ;
		}
	}

	//TSobt@SĔj
	for( int i=0; i<4; ++i ) {
		if( dataIoTs[i] ) {
			dataIoTs[i]->ClearBuff();
		}
	}
	//}CNpPbgJ[\NA
	for( int i=0; i<4; ++i ) {
		m_rawPacketCur[i] = 0;
	}

	Sleep(100);

	// DMA ]
	enStatus = m_pcDevice->SetTransferEnable(true);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
}

void CDataIO::Stop()
{
	if( m_hThread != NULL ){
		m_bQuit = true;
		::SetEvent(m_hStopEvent);
		// XbhI҂
		if ( ::WaitForSingleObject(m_hThread, 2000) == WAIT_TIMEOUT ){
			::TerminateThread(m_hThread, 0xffffffff);
		}
		CloseHandle(m_hThread);
		m_hThread = NULL;
	}
}

void CDataIO::EnableTuner(int iID, BOOL bEnable, DWORD unique)
{
	int iDevID = iID>>16;
	PT::Device::ISDB enISDB = (PT::Device::ISDB)((iID&0x0000FF00)>>8);
	uint iTuner = iID&0x000000FF;

	int id = GetID( enISDB, iTuner ); 
	DATAIO_TS *&ts = dataIoTs[id];

	if( bEnable == TRUE ){
		if( !ts ) ts = new DATAIO_TS;
		ts->callBack_Object.id = id;
		ts->callBack_Object.iid = iID;
		ts->callBack_Object.dataIO = this;
		m_SetBuff[id]->dwSetSize = 0;

		wstring strName = L"";
		Format(strName, L"%s%08x%08x", CMD_PT1_DATA, iID, unique );
		ts->m_cMem = new CMemServer( strName.c_str() );
		ts->m_cMem->StartServer( OutsideCallback, (void *)&ts->callBack_Object, THREAD_PRIORITY_ABOVE_NORMAL);
	} else {
		m_SetBuff[id]->dwSetSize = 0;
		ts->m_cMem->StopServer();
		delete ts->m_cMem;
		ts->m_cMem = NULL;
		delete ts;
		ts = NULL;
	}
}

UINT WINAPI CDataIO::RecvThread(LPVOID pParam)
{
	CDataIO* pSys = (CDataIO*)pParam;

	HANDLE hCurThread = GetCurrentThread();
	SetThreadPriority(hCurThread, THREAD_PRIORITY_HIGHEST);

	pSys->mVirtualIndex = 0;
	pSys->mImageIndex = 0;
	pSys->mBlockIndex = 0;

	while (true) {
		DWORD dwRes = WaitForSingleObject(pSys->m_hStopEvent, 0);
		if( dwRes == WAIT_OBJECT_0 ){
			//STOP
			break;
		}

		bool b;
		
		b = pSys->WaitBlock();
		if (b == false) break;

		pSys->CopyBlock();

		b = pSys->DispatchBlock();
		if (b == false) break;
	}

	return 0;
}

// 1ubN DMA ]I邩 mQuit  true ɂȂ܂ő҂
bool CDataIO::WaitBlock()
{
	bool b = true;

	while (true) {
		if (m_bQuit) {
			b = false;
			break;
		}

		// ubN̖ 0 łȂ΁ÃubN DMA ]ƂɂȂ
		if (Read(mVirtualIndex, mImageIndex, mBlockIndex) != 0) break;
		Sleep(5);
	}
	//::wprintf(L"(mVirtualIndex, mImageIndex, mBlockIndex) = (%d, %d, %d) ̓]I܂B\n", mVirtualIndex, mImageIndex, mBlockIndex);

	return b;
}

// 1ubÑf[^e|̈ɃRs[BCPU 猩 DMA obt@̓LbVȂ߁A
// LbV̈ɃRs[ĂANZXƌ܂܂B
void CDataIO::CopyBlock()
{
	status enStatus;

	void *voidPtr;
	enStatus = m_pcDevice->GetBufferPtr(mVirtualIndex, &voidPtr);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}
	DWORD dwOffset = ((TRANSFER_SIZE*mImageIndex) + (READ_BLOCK_SIZE*mBlockIndex));

	memcpy(m_bDMABuff, (BYTE*)voidPtr+dwOffset, READ_BLOCK_SIZE);

	// Rs[ÎŁAubN̖ 0 ɂ܂B
	uint *ptr = static_cast<uint *>(voidPtr);
	ptr[Offset(mImageIndex, mBlockIndex, READ_BLOCK_SIZE)-1] = 0;

	if (READ_BLOCK_COUNT <= ++mBlockIndex) {
		mBlockIndex = 0;

		// ]JE^ OS::Memory::PAGE_SIZE * PT::Device::BUFFER_PAGE_COUNT oCgƂɃCNg܂B
		enStatus = m_pcDevice->IncrementTransferCounter();
		if( enStatus != PT::STATUS_OK ){
			return ;
		}

		if (VIRTUAL_IMAGE_COUNT <= ++mImageIndex) {
			mImageIndex = 0;
			if (VIRTUAL_COUNT <= ++mVirtualIndex) {
				mVirtualIndex = 0;
			}
		}
	}
}

void CDataIO::Clear(uint virtualIndex, uint imageIndex, uint blockIndex)
{
	void *voidPtr;
	status enStatus = m_pcDevice->GetBufferPtr(virtualIndex, &voidPtr);
	if( enStatus != PT::STATUS_OK ){
		return ;
	}

	uint *ptr = static_cast<uint *>(voidPtr);
	ptr[Offset(imageIndex, blockIndex, READ_BLOCK_SIZE)-1] = 0;
}

uint CDataIO::Read(uint virtualIndex, uint imageIndex, uint blockIndex) const
{
	void *voidPtr;
	status enStatus = m_pcDevice->GetBufferPtr(virtualIndex, &voidPtr);
	if( enStatus != PT::STATUS_OK ){
		return 0;
	}

	volatile const uint *ptr = static_cast<volatile const uint *>(voidPtr);
	return ptr[Offset(imageIndex, blockIndex, READ_BLOCK_SIZE)-1];
}

uint CDataIO::Offset(uint imageIndex, uint blockIndex, uint additionalOffset) const
{
	uint offset = ((TRANSFER_SIZE*imageIndex) + (READ_BLOCK_SIZE*blockIndex) + additionalOffset) / sizeof(uint);

	return offset;
}
//#define _SSE2
#ifdef _SSE2
bool CDataIO::DispatchBlock()
{
	const DWORD *ptr = (DWORD*)m_bDMABuff;

	for (uint i=0; i<READ_BLOCK_SIZE / 4; ++i) {
		_mm_prefetch( (const char *)ptr + 32, _MM_HINT_NTA );
		DWORD data = *ptr++;
		uint packetStart = BIT_SHIFT_MASK( data, 25, 1);
		uint packetId = BIT_SHIFT_MASK(data, 29,  3) - 1;
		if( packetId >= 4 ) continue;

		if( packetStart ) {
			m_rawPacketCur[packetId] = 0;
		}
		(*m_rawPacket)[packetId][ m_rawPacketCur[packetId]++ ] = _byteswap_ulong (  data  );
		if( m_rawPacketCur[packetId] == 189/3 ) {
			m_rawPacketCur[packetId] = 0;
			if( MicroPacket2( packetId ) == false ) {
				DispError();
				break;
			}
		}
	}

	return true;
}

//słꍇtrue słȂꍇfalse
bool CDataIO::MicroPacket2(DWORD packetId)
{

	int id;
	switch(packetId){
		case 0: //S0
			id = 2;
			break;
		case 1: //T0
			id = 0;
			break;
		case 2: //S1
			id = 3;
			break;
		case 3: //T1
			id = 1;
			break;
		default:
			return true;
	}
	DWORD *data = (DWORD *)(*m_rawPacket)[packetId];
	BYTE *dest = m_SetBuff[id]->pbBuff + m_SetBuff[id]->dwSetSize;

	//rawPacketTSpPbgւ̕ϊ	
	__asm {
		pcmpeqb xmm6, xmm6
		movq xmm7, xmm6
		pxor xmm5, xmm5		 //zero
		pxor xmm7, xmm6      //mask2 { 0xffffffff, 0xffffffff, 0, 0 };
		punpckhdq xmm6, xmm5 //mask1 { 0, 0xffffffff, 0, 0xffffffff };

		mov esi, dword ptr[data]
		mov edi, dword ptr[dest]

		mov ecx, 0xf //15񃋁[v
LOOP1:
		prefetchnta [esi+16]
		prefetcht0 [edi+12]


		movdqa xmm0, [esi] 

		psrld  xmm0, 8	  //DWORD*4 WrbgVtg
		movdqa xmm1, xmm6
		movdqa xmm2, xmm6
		pandn  xmm2, xmm0 //d0b0
		pand   xmm1, xmm0 //0c0a
		psrldq xmm2, 1    //1oCgVtg
		por    xmm1, xmm2 //dcba

		movdqa xmm3, xmm7
		movdqa xmm4, xmm7
		pand   xmm3, xmm1
		pandn  xmm4, xmm1
		psrldq xmm3, 2    //2oCgVtg
		por    xmm3, xmm4

		movdqu [edi], xmm3

		add esi, 16			
		add edi, 12

		loop LOOP1


		mov dword ptr[data], esi
		mov dword ptr[dest], edi
	}

	DWORD dat;
	dat = *data++;
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;

	dat = *data++;
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;

	dat = *data;
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;


	//uint packetCount = BIT_SHIFT_MASK(data[3], 2,  3);
	//uint packetError = BIT_SHIFT_MASK(data[3], 0,  1);
	//uint packetError = BIT_SHIFT_MASK(*data, 0,  1);
	uint packetError = BIT_SHIFT_MASK(*data, 0,  1);
	if( packetError ) {
		return false;
	}
	if( *(m_SetBuff[id]->pbBuff + m_SetBuff[id]->dwSetSize) != 0x47 ) {
		//TSpPbgƂĔF߂Ȃ
		return true;
	}

	m_SetBuff[id]->dwSetSize+=188;
	if( m_SetBuff[id]->dwSetSize >= m_SetBuff[id]->dwSize ){
		if( !dataIoTs[id] ) {
			m_SetBuff[id]->dwSetSize = 0;
			return true;
		}
		if( dataIoTs[id]->PushBuff( m_SetBuff[id] ) ) {
			m_SetBuff[id] = new BUFF_DATA;
		}
	}
	return true;
}
#else 

bool CDataIO::DispatchBlock()
{
	const DWORD *ptr = (DWORD*)m_bDMABuff;

	for (uint i=0; i<READ_BLOCK_SIZE / 4; ++i) {
		_mm_prefetch( (const char *)ptr + 32, _MM_HINT_NTA );
		DWORD data = *ptr++;
		uint packetStart = BIT_SHIFT_MASK( data, 25, 1);
		uint packetId = BIT_SHIFT_MASK(data, 29,  3) - 1;
		if( packetId >= 4 ) continue;

		if( packetStart ) {
			m_rawPacketCur[packetId] = 0;
		}
		(*m_rawPacket)[packetId][ m_rawPacketCur[packetId]++ ] = data;
		if( m_rawPacketCur[packetId] == 189/3 ) {
			m_rawPacketCur[packetId] = 0;
			if( MicroPacket2( packetId ) == false ) {
				DispError();
				break;
			}
		}
	}

	return true;
}

//słꍇtrue słȂꍇfalse
bool CDataIO::MicroPacket2(DWORD packetId)
{

	int id;
	switch(packetId){
		case 0: //S0
			id = 2;
			break;
		case 1: //T0
			id = 0;
			break;
		case 2: //S1
			id = 3;
			break;
		case 3: //T1
			id = 1;
			break;
		default:
			return true;
	}
	DWORD *data = (DWORD *)(*m_rawPacket)[packetId];
	BYTE *dest = m_SetBuff[id]->pbBuff + m_SetBuff[id]->dwSetSize;
	register DWORD dat;
	//rawPacketTSpPbgւ̕ϊ
	for( int i=0; i< 15; ++i ) { // 12*15 = 180
		_mm_prefetch( (const char *)data+16, _MM_HINT_NTA );
		_mm_prefetch( (const char *)dest+12, _MM_HINT_T0);

		dat = _byteswap_ulong(*data++);
		dat >>= 8;
		*dest++ = (BYTE)dat;
		dat >>= 8;
		*dest++ = (BYTE)dat;
		dat >>= 8;
		*dest++ = (BYTE)dat;

		dat = _byteswap_ulong(*data++);
		dat >>= 8;
		*dest++ = (BYTE)dat;
		dat >>= 8;
		*dest++ = (BYTE)dat;
		dat >>= 8;
		*dest++ = (BYTE)dat;

		dat = _byteswap_ulong(*data++);
		dat >>= 8;
		*dest++ = (BYTE)dat;
		dat >>= 8;
		*dest++ = (BYTE)dat;
		dat >>= 8;
		*dest++ = (BYTE)dat;

		dat = _byteswap_ulong(*data++);
		dat >>= 8;
		*dest++ = (BYTE)dat;
		dat >>= 8;
		*dest++ = (BYTE)dat;
		dat >>= 8;
		*dest++ = (BYTE)dat;
	}

	dat = _byteswap_ulong(*data++);
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;

	dat = _byteswap_ulong(*data++);
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;

	dat = _byteswap_ulong(*data);
	dat >>= 8;
	*dest++ = (BYTE)dat;
	dat >>= 8;
	*dest++ = (BYTE)dat;


	//uint packetCount = BIT_SHIFT_MASK(data[3], 2,  3);
	uint packetError = BIT_SHIFT_MASK(*data, 24,  1);
	if( packetError ) {
		return false;
	}
	if( *(m_SetBuff[id]->pbBuff + m_SetBuff[id]->dwSetSize) != 0x47 ) {
		//TSpPbgƂĔF߂Ȃ
		return true;
	}

	m_SetBuff[id]->dwSetSize+=188;
	if( m_SetBuff[id]->dwSetSize >= m_SetBuff[id]->dwSize ){
		if( !dataIoTs[id] ) {
			m_SetBuff[id]->dwSetSize = 0;
			return true;
		}
		if( dataIoTs[id]->PushBuff( m_SetBuff[id] ) ) {
			m_SetBuff[id] = new BUFF_DATA;
		}
	}
	return true;
}

#endif

void CDataIO::DispError() {
	// G[̌𒲂ׂ
	PT::Device::TransferInfo info;
	status enStatus = m_pcDevice->GetTransferInfo(&info);
	if( enStatus == PT::STATUS_OK ){
		if (info.TransferCounter0) {
			OutputDebugString(L"]JE^ 0 ł̂oB");
			ResetDMA();
			return;
		} else if (info.TransferCounter1) {
			OutputDebugString(L"]JE^ 1 ȉɂȂ܂B");
			ResetDMA();
			return;
		} else if (info.BufferOverflow) {
			OutputDebugString(L"PCI oX𒷊ɓnmۂłȂ߁A{[h FIFO ܂B");
			ResetDMA();
			return;
		} else {
			OutputDebugString(L"]G[܂B");
			return;
		}
	}else{
		OutputDebugString(L"GetTransferInfo() err");
		return;
	}
}

bool CDataIO::DATAIO_TS::PushBuff( BUFF_DATA* data)
{
	bool bRet = true;
	Lock();

	if( m_Buff.size() > MAX_DATA_BUFF_COUNT ){
		data->dwSetSize = 0;

		m_dwOverFlowCount++;
		WCHAR str[64];
		wsprintf( str, L"Buff Full ID%02d Count=%d", callBack_Object.id, m_dwOverFlowCount );
		OutputDebugString( str );
		bRet = false;
	}else{
		m_Buff.push( data );
		m_dwOverFlowCount = 0;
		bRet = true;
	}
	UnLock();
	return bRet;
}


int CALLBACK CDataIO::OutsideCallback(void *pParam, void *buf )
{
	CALLBACK_OBJECT *pObj = (CALLBACK_OBJECT *)pParam;
	CDataIO* pSys = (CDataIO*)pObj->dataIO;
	DATAIO_TS &ts = *pSys->dataIoTs[pObj->id];

	ts.Lock();
	int count = ts.m_Buff.size();
	count = min( count, 8 );
	BYTE *p = (BYTE *)buf +64;
	for(int i=0;i<count; ++i ) {
		BUFF_DATA *data = ts.m_Buff.front();
		ts.m_Buff.pop();
		memcpy( p, data->pbBuff, DATA_BUFF_SIZE );
		p += DATA_BUFF_SIZE;
		SAFE_DELETE(data);
	}
	*(DWORD *)buf = count;
	ts.UnLock();

	return 0;
}

void CDataIO::CmdSendData(DWORD dwID, CMD_STREAM* pCmdParam, CMD_STREAM* pResParam)
{
	OutputDebugString( L"CmdSendData\n" );
	pResParam->dwParam = CMD_SUCCESS;
	BOOL bSend = FALSE;
	
	DATAIO_TS &ts = *dataIoTs[dwID];

	ts.Lock();
	if( ts.m_Buff.size() > 0 ){
		BUFF_DATA *data = ts.m_Buff.front();
		ts.m_Buff.pop();

		pResParam->dwSize = data->dwSize;
		pResParam->bData = data->pbBuff;
		bSend = TRUE;
		data->dwSize = 0;
		data->pbBuff = 0;
		SAFE_DELETE(data);
	}
	ts.UnLock();

	if( bSend == FALSE ){
		pResParam->dwParam = CMD_ERR_BUSY;
	}
}

DWORD CDataIO::GetOverFlowCount(int iID)
{
	int iDevID = iID>>16;
	PT::Device::ISDB enISDB = (PT::Device::ISDB)((iID&0x0000FF00)>>8);
	uint iTuner = iID&0x000000FF;

	int id = GetID( enISDB, iTuner );
	DWORD dwRet = dataIoTs[id]->m_dwOverFlowCount;

	return dwRet;
}


