#pragma once

#include "inc/EARTH_PT.h"
#include "inc/Prefix.h"
#include "../../Common/PT1OutsideCtrlCmdDef.h"
#include "../../Common/PipeServer.h"
#include "../../Common/MemServer.h"

#define TRANSFER_SIZE (4096*PT::Device::BUFFER_PAGE_COUNT)
#define VIRTUAL_IMAGE_COUNT 4
#define LOCK_SIZE 4
#define READ_BLOCK_COUNT 8
#define READ_BLOCK_SIZE (TRANSFER_SIZE / READ_BLOCK_COUNT)

using namespace EARTH;

class CDataIO
{
public:
	CDataIO(void);
	~CDataIO(void);

	void SetDevice(PT::Device* pcDevice){ m_pcDevice = pcDevice; };
	void SetVirtualCount(UINT uiVirtualCount){ VIRTUAL_COUNT = uiVirtualCount; };
	void Run();
	void Stop();
	void EnableTuner(int iID, BOOL bEnable, DWORD unique);
	void ClearBuff(int iID);

	DWORD GetOverFlowCount(int iID);

protected:
	UINT VIRTUAL_COUNT;
	PT::Device* m_pcDevice;

	uint mVirtualIndex;
	uint mImageIndex;
	uint mBlockIndex;
	volatile bool m_bQuit;

	HANDLE m_hStopEvent;
	HANDLE m_hThread;

	struct BUFF_DATA{
		BYTE* pbBuff;
		DWORD dwSize;
		DWORD dwSetSize;
		BUFF_DATA(void){
			dwSize = DATA_BUFF_SIZE;
			pbBuff = NEW_DATA_BUFF(dwSize);
			dwSetSize = 0;
		}
		~BUFF_DATA(void){
			DELETE_DATA_BUFF(pbBuff, dwSize);
		}
		void *operator new(size_t n ) {
			return NEW_DATA_BUFF( n );
		}
		void operator delete(void *p) {
			BYTE *b = (BYTE *)p;
			DELETE_DATA_BUFF( b, sizeof(BUFF_DATA) );
		}
	};

	struct CALLBACK_OBJECT {
		CDataIO *dataIO;
		int id;
		int iid;
	};

	struct DATAIO_TS {
		DATAIO_TS() {
			m_hEvent = _CreateEvent(FALSE, TRUE, NULL );
			m_dwOverFlowCount = 0;
			callBack_Object.dataIO = NULL;
			callBack_Object.id = -1;
			InitializeCriticalSection( &m_csBuff );
		}
		~DATAIO_TS() {
			while( !m_Buff.empty() ) {
				SAFE_DELETE(m_Buff.front() );
				m_Buff.pop();
			}
			if( m_hEvent != NULL ){
				UnLock();
				CloseHandle(m_hEvent);
				m_hEvent = NULL;
			}
			DeleteCriticalSection( &m_csBuff );
		}
		void Lock() {
			EnterCriticalSection( &m_csBuff );
		}
		void UnLock() {
			LeaveCriticalSection( &m_csBuff );
		}
		
		void ClearBuff();

		CMemServer *m_cMem;
		queue<BUFF_DATA*> m_Buff;
		DWORD m_dwOverFlowCount;
		CRITICAL_SECTION m_csBuff;
		bool PushBuff( BUFF_DATA* data);
		HANDLE m_hEvent;

		CALLBACK_OBJECT callBack_Object;
	} *dataIoTs[4]; // T0=0 T1=1 S0=2 S1=3

	//`[i[Ă}CNpPbgׂ͂
	//DATAIO_TSɑgݍ܂Ȃ
	DWORD m_rawPacketCur[4];
	DWORD (*m_rawPacket)[4][256/4];

	BUFF_DATA* m_SetBuff[4];

	int GetID(enum PT::Device::ISDB enISDB, uint iTuner ) {
		if( enISDB == PT::Device::ISDB_T ){
			if( iTuner == 0 ){
				return 0;
			}else{
				return 1;
			}
		}else{
			if( iTuner == 0 ){
				return 2;
			}else{
				return 3;
			}
		}
	}

	BYTE* m_bDMABuff;

protected:
	static UINT WINAPI RecvThread(LPVOID pParam);

	bool WaitBlock();
	void CopyBlock();
	bool DispatchBlock();
	void DispError();
	void Clear(uint virtualIndex, uint imageIndex, uint blockIndex);
	uint Read(uint virtualIndex, uint imageIndex, uint blockIndex) const;
	uint Offset(uint imageIndex, uint blockIndex, uint additionalOffset = 0) const;
	void MicroPacket(DWORD dwPacket);
	bool MicroPacket2(DWORD dwID);

	static int CALLBACK OutsideCmdCallback(void* pParam, CMD_STREAM* pCmdParam, CMD_STREAM* pResParam);
	static int CALLBACK OutsideCallback(void* pParam, void *buf );

	void CmdSendData(DWORD dwID, CMD_STREAM* pCmdParam, CMD_STREAM* pResParam);

	void ResetDMA();

};
